# Authors: Clemens Brunner <clemens.brunner@gmail.com>
#
# License: BSD (3-clause)

from .mainwindow import MainWindow, __version__  # noqa: F401
from .model import Model


__all__ = [MainWindow, Model]
